@@ -25,8 +25,8 @@
  *
  ************************************************************************/
 
-#ifndef __MDDS_MULTI_TYPE_VECTOR_HPP__
-#define __MDDS_MULTI_TYPE_VECTOR_HPP__
+#ifndef MDDS_MULTI_TYPE_VECTOR_HPP
+#define MDDS_MULTI_TYPE_VECTOR_HPP
 
 #include "default_deleter.hpp"
 #include "compat/unique_ptr.hpp"
@@ -38,6 +38,14 @@
 #include <algorithm>
 #include <cassert>
 
+#if defined(MDDS_UNIT_TEST) || defined (MDDS_MULTI_TYPE_VECTOR_DEBUG)
+#include <sstream>
+#include <iostream>
+using std::cout;
+using std::cerr;
+using std::endl;
+#endif
+
 namespace mdds {
 
 /**
@@ -205,7 +213,7 @@ public:
      *         inserted.
      */
     template<typename _T>
-    iterator set(iterator pos_hint, size_type pos, const _T& value);
+    iterator set(const iterator& pos_hint, size_type pos, const _T& value);
 
     /**
      * Set multiple values of identical type to a range of elements starting
@@ -269,7 +277,7 @@ public:
      *         is empty, the end iterator position is returned.
      */
     template<typename _T>
-    iterator set(iterator pos_hint, size_type pos, const _T& it_begin, const _T& it_end);
+    iterator set(const iterator& pos_hint, size_type pos, const _T& it_begin, const _T& it_end);
 
     /**
      * Insert multiple values of identical type to a specified position.
@@ -333,7 +341,7 @@ public:
      *         is empty, the end iterator position is returned.
      */
     template<typename _T>
-    iterator insert(iterator pos_hint, size_type pos, const _T& it_begin, const _T& it_end);
+    iterator insert(const iterator& pos_hint, size_type pos, const _T& it_begin, const _T& it_end);
 
     /**
      * Get the value of an element at specified position.  The caller must
@@ -356,12 +364,133 @@ public:
      * <p>The method will throw an <code>std::out_of_range</code> exception if
      * the specified position is outside the current container range.</p>
      *
-     * @param position position of the element value to retrieve.
+     * @param pos position of the element value to retrieve.
      * @return element value.
      */
     template<typename _T>
     _T get(size_type pos) const;
 
+    /**
+     * Return the value of an element at specified position and set that
+     * position empty.  If the element resides in a managed block, this call
+     * will <i>not</i> delete that element.  If the element is on a
+     * non-managed block, this call is equivalent to set_empty(pos, pos).
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos position of the element to release.
+     *
+     * @return element value.
+     */
+    template<typename _T>
+    _T release(size_type pos);
+
+    /**
+     * Given the logical position of an element, get the iterator of the block
+     * where the element is located, and its offset from the first element of
+     * that block.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos logical position of the element.
+     * @return iterator referencing the block where the element resides, and
+     *         its offset within the block.
+     */
+    std::pair<iterator, size_type> position(size_type pos);
+
+    /**
+     * Given the logical position of an element, get the iterator of the block
+     * where the element is located, and its offset from the first element of
+     * that block.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos_hint iterator used as a block position hint, to specify
+     *                 which block to start when searching for the element
+     *                 position.
+     * @param pos logical position of the element.
+     * @return iterator referencing the block where the element resides, and
+     *         its offset within the block.
+     */
+    std::pair<iterator, size_type> position(const iterator& pos_hint, size_type pos);
+
+    /**
+     * Given the logical position of an element, get the iterator of the block
+     * where the element is located, and its offset from the first element of
+     * that block.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos position of the element.
+     * @return iterator referencing the block where the element resides, and
+     *         its offset within the block.
+     */
+    std::pair<const_iterator, size_type> position(size_type pos) const;
+
+    /**
+     * Move elements from one container to another. After the move, the
+     * segment where the elements were in the original container becomes
+     * empty.  When transferring managed elements, this call transfers
+     * ownership of the moved elements to the new container.  The moved
+     * elements will overwrite any existing elements in the destination range.
+     * Transfer of elements within the same container is not allowed.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * either the starting or the ending position is outside the current
+     * container size, or the destination container is not large enough to
+     * accommodate the transferred elements.</p>
+     *
+     * @param start_pos starting position
+     * @param end_pos ending position, inclusive.
+     * @param dest destination container to which the elements are to be
+     *             moved.
+     * @param dest_pos position in the destination container to which the
+     *                 elements are to be moved.
+     *
+     * @return iterator referencing the block where the moved elements were
+     *         prior to the transfer.
+     */
+    iterator transfer(size_type start_pos, size_type end_pos, multi_type_vector& dest, size_type dest_pos);
+
+    /**
+     * Move elements from one container to another. After the move, the
+     * segment where the elements were in the original container becomes
+     * empty.  When transferring managed elements, this call transfers
+     * ownership of the moved elements to the new container.  The moved
+     * elements will overwrite any existing elements in the destination range.
+     * Transfer of elements within the same container is not allowed.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * either the starting or the ending position is outside the current
+     * container size, or the destination container is not large enough to
+     * accommodate the transferred elements.</p>
+     *
+     * @param pos_hint iterator used as a block position hint, to specify
+     *                 which block to start when searching for the blocks
+     *                 where the elements to be transferred reside.
+     * @param start_pos starting position
+     * @param end_pos ending position, inclusive.
+     * @param dest destination container to which the elements are to be
+     *             moved.
+     * @param dest_pos position in the destination container to which the
+     *                 elements are to be moved.
+     *
+     * @return iterator referencing the block where the moved elements were
+     *         prior to the transfer.
+     */
+    iterator transfer(const iterator& pos_hint, size_type start_pos, size_type end_pos, multi_type_vector& dest, size_type dest_pos);
+
+    /**
+     * Get the type of an element at specified position.
+     *
+     * @param pos position of the element.
+     *
+     * @return element type.
+     */
     mtv::element_t get_type(size_type pos) const;
 
     /**
@@ -420,7 +549,7 @@ public:
      * @return iterator position pointing to the block where the elements are
      *         emptied.
      */
-    iterator set_empty(iterator pos_hint, size_type start_pos, size_type end_pos);
+    iterator set_empty(const iterator& pos_hint, size_type start_pos, size_type end_pos);
 
     /**
      * Erase elements located between specified start and end positions. The
@@ -488,7 +617,7 @@ public:
      *         is inserted. When no insertion occurs because the length is
      *         zero, the end iterator position is returned.
      */
-    iterator insert_empty(iterator pos_hint, size_type pos, size_type length);
+    iterator insert_empty(const iterator& pos_hint, size_type pos, size_type length);
 
     /**
      * Clear the content of the container.  The size of the container will
@@ -557,6 +686,12 @@ public:
     template<typename _T>
     static mtv::element_t get_element_type(const _T& elem);
 
+#ifdef MDDS_MULTI_TYPE_VECTOR_DEBUG
+    void dump_blocks(std::ostream& os) const;
+
+    bool check_block_integrity() const;
+#endif
+
 private:
 
     template<typename _T>
@@ -587,7 +722,7 @@ private:
      * Same as above, but try to infer block position from the iterator first
      * before trying full search.
      */
-    void get_block_position(iterator pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
+    void get_block_position(const iterator& pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
 
     template<typename _T>
     static void create_new_block_with_new_cell(element_block_type*& data, const _T& cell);
@@ -615,19 +750,30 @@ private:
     void set_cell_to_bottom_of_data_block(
         size_type block_index, const _T& cell);
 
+    iterator transfer_impl(
+        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
+        multi_type_vector& dest, size_type dest_pos);
+
+    iterator transfer_single_block(
+        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
+        multi_type_vector& dest, size_type dest_pos);
+
     iterator set_empty_impl(size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1);
 
-    iterator set_whole_block_empty(size_type block_index, size_type start_pos_in_block);
+    iterator set_whole_block_empty(size_type block_index, size_type start_pos_in_block, bool overwrite);
 
     iterator set_empty_in_single_block(
-        size_type start_pos, size_type end_pos, size_type block_index, size_type start_pos_in_block);
+        size_type start_pos, size_type end_pos, size_type block_index, size_type start_pos_in_block,
+        bool overwrite);
 
     iterator set_empty_in_multi_blocks(
         size_type start_pos, size_type end_pos,
         size_type block_index1, size_type start_pos_in_block1,
-        size_type block_index2, size_type start_pos_in_block2);
+        size_type block_index2, size_type start_pos_in_block2, bool overwrite);
 
     void erase_impl(size_type start_pos, size_type end_pos);
+    void erase_in_single_block(
+        size_type start_pos, size_type end_pos, size_type block_pos, size_type start_pos_in_block);
 
     iterator insert_empty_impl(size_type row, size_type start_pos, size_type block_index, size_type length);
 
@@ -668,7 +814,24 @@ private:
         size_type block_index2, size_type start_pos_in_block2,
         const _T& it_begin, const _T& it_end);
 
-    void merge_with_next_block(size_type block_index);
+    /**
+     * Merge with previous or next block as needed.
+     *
+     * @param block_index index of the block that may need merging.
+     *
+     * @return size of previous block if the block is merged with the previous
+     *         block, or 0 if it didn't merge with the previous block.
+     */
+    size_type merge_with_adjacent_blocks(size_type block_index);
+
+    /**
+     * Merge only with the next block if the two are of the same type.
+     *
+     * @param block_index index of the block that may need merging.
+     *
+     * @return true if merge occurs, false otherwise.
+     */
+    bool merge_with_next_block(size_type block_index);
 
     template<typename _T>
     bool append_to_prev_block(
