@@ -40,7 +40,7 @@
 using namespace std;
 using namespace mdds;
 
-void printTitle(const char* msg)
+void print_title(const char* msg)
 {
     cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
     cout << " " << msg << endl;
@@ -50,7 +50,7 @@ void printTitle(const char* msg)
 void fst_test_leaf_search()
 {
     {
-        printTitle("Simple insert test");
+        print_title("Simple insert test");
         flat_segment_tree<int, int> int_ranges(0, 100, -1);
         for (int i = 0; i < 20; ++i)
         {
@@ -62,7 +62,7 @@ void fst_test_leaf_search()
     }
 
     {
-        printTitle("Merge test 1");
+        print_title("Merge test 1");
         flat_segment_tree<int, int> merge_test(0, 100, -1);
         merge_test.insert_front(10, 20, 5);
         merge_test.dump_leaf_nodes();
@@ -77,7 +77,7 @@ void fst_test_leaf_search()
     }
 
     {
-        printTitle("Merge test 2");
+        print_title("Merge test 2");
         flat_segment_tree<int, int> merge_test(0, 100, -1);
 
         // This should not change the node configuration.
@@ -104,7 +104,7 @@ void fst_test_leaf_search()
     }
 
     {
-        printTitle("Search test");
+        print_title("Search test");
         flat_segment_tree<int, int> db(0, 100, -1);
         for (int i = 0; i < 10; ++i)
         {
@@ -179,7 +179,7 @@ void fst_perf_test_search(bool tree_search)
         int val;
         for (int i = lower; i < upper; ++i)
         {
-            if (db.search_tree(i, val))
+            if (db.search_tree(i, val).second)
                 ++success;
             else
                 ++failure;
@@ -193,7 +193,7 @@ void fst_perf_test_search(bool tree_search)
         {
             if (tree_search)
             {
-                if (db.search_tree(i, val))
+                if (db.search_tree(i, val).second)
                     ++success;
                 else
                     ++failure;
@@ -213,8 +213,9 @@ void fst_perf_test_search(bool tree_search)
 void fst_test_tree_search()
 {
     stack_printer __stack_printer__("::fst_test_tree_search");
+    typedef flat_segment_tree<int, int> fst_type;
     int lower = 0, upper = 200, delta = 5;
-    flat_segment_tree<int, int> db(lower, upper, 0);
+    fst_type db(lower, upper, 0);
     for (int i = lower; i < upper; i += delta)
         db.insert_front(i, i+delta, i);
 
@@ -226,7 +227,7 @@ void fst_test_tree_search()
     int success = 0, failure = 0;
     for (int i = lower-10; i < upper+10; ++i)
     {
-        if (db.search_tree(i, val, &start, &end))
+        if (db.search_tree(i, val, &start, &end).second)
         {
             cout << "key = " << i << "; value = " << val << " (" << start << "-" << end << ")" << endl;
             ++success;
@@ -238,12 +239,48 @@ void fst_test_tree_search()
         }
     }
     cout << "search: success (" << success << ")  failure (" << failure << ")" << endl;
+
+    // Make sure search_tree() returns correct iterator position.
+    db.clear();
+    db.insert_back(5, 10, 2);
+    db.insert_back(15, 18, 3);
+    db.insert_back(23, 28, 4);
+    db.build_tree();
+
+    typedef pair<fst_type::const_iterator,bool> ret_type;
+    ret_type ret = db.search_tree(0, val, &start, &end);
+    assert(ret.second);
+    assert(start == 0 && end == 5 && val == 0);
+    assert(ret.first == db.begin());
+
+    ret = db.search_tree(6, val, &start, &end);
+    assert(ret.second);
+    assert(start == 5 && end == 10 && val == 2);
+    fst_type::const_iterator check = db.begin();
+    ++check; // 5-10 is the 2nd segment from the top.
+    assert(ret.first == check);
+
+    ret = db.search_tree(17, val, &start, &end);
+    assert(ret.second);
+    assert(start == 15 && end == 18 && val == 3);
+    std::advance(check, 2);
+    assert(ret.first == check);
+
+    ret = db.search_tree(55, val, &start, &end);
+    assert(ret.second);
+    assert(start == 28 && end == upper && val == 0);
+    std::advance(check, 3);
+    assert(ret.first == check);
+
+    ret = db.search_tree(upper+10, val, &start, &end);
+    assert(!ret.second); // This search should fail.
+    assert(ret.first == db.end());
 }
 
 void test_single_tree_search(const flat_segment_tree<int, int>& db, int key, int val, int start, int end)
 {
     int r_val, r_start, r_end;
-    if (db.search_tree(key, r_val, &r_start, &r_end))
+    if (db.search_tree(key, r_val, &r_start, &r_end).second)
         assert(r_val == val && r_start == start && r_end == end);
     else
         assert(!"tree search failed!");
@@ -1764,7 +1801,7 @@ void fst_test_swap()
 
     // Tree search should work on db2.
     db_type::value_type val;
-    assert(db2.search_tree(35, val));
+    assert(db2.search_tree(35, val).second);
     assert(val == 2);
 }
 
